/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */

#import <Foundation/Foundation.h>

/**
 * Javaのjava.nio.ByteBufferのような機能を提供します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/08/12
 */
@interface ByteBuffer : NSObject {
    NSData*     _source;    /**< 元データ */
    uint8_t*    _head;      /**< 先頭のポインタ */
    uint32_t    _pos;       /**< 現在位置 */
    uint32_t    _limit;     /**< 終端位置 */
}
@property(readonly) NSData* source;

/**
 * 初期化します。
 * @param   [in]    data    操作するデータ
 * @return  自身を返します。
 */
- (id)initWithData:(NSData*)data;

/**
 * Boolean値を取得します。
 * @return  現在位置から1バイトをBoolean値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (Boolean)getBoolean;

/**
 * Boolean値を取得します。
 * @return  現在位置から4バイトをBoolean値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (Boolean)getBooleanWithInt;

/**
 * Byte値を取得します。
 * @return  現在位置から1バイトをByte値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (uint8_t)getByte;

/**
 * バイトデータを取得します。
 * @param   [in]    size    読み取るバイト数
 * @return  現在位置から指定されたバイト数分を返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSData*)getBytes:(uint32_t)size;

/**
 * 1970/01/01 00:00:00からの経過時間を取得します。
 * @return  現在位置から8バイトを1970/01/01 00:00:00からの経過ミリ秒として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSDate*)getDateSince1970WithMillis;

/**
 * 1970/01/01 00:00:00からの経過時間を取得します。
 * @return  現在位置から8バイトを1970/01/01 00:00:00からの経過秒として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSDate*)getDateSince1970WithSeconds;

/**
 * 1970/01/01 00:00:00からの経過時間を取得します。
 * @attention   このメソッドでは、日本標準時から協定世界時(UTC)に変換します。
 * @return  現在位置から8バイトを1970/01/01 00:00:00からの経過ミリ秒として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSDate*)getDateSince1970WithLocalMillis;

/**
 * 1970/01/01 00:00:00からの経過時間を取得します。
 * @attention   このメソッドでは、日本標準時から協定世界時(UTC)に変換します。
 * @return  現在位置から8バイトを1970/01/01 00:00:00からの経過秒として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSDate*)getDateSince1970WithLocalSeconds;

/**
 * 32ビット整数値を取得します。
 * @return  現在位置から4バイトを32ビット整数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (int32_t)getInt;

/**
 * Base128形式で32ビット整数値を取得します。
 * @return  現在位置から4バイトを32ビット整数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (int32_t)getIntUsingBase128;

/**
 * 32ビット符号なし整数値を取得します。
 * @return  現在位置から4バイトを32ビット符号なし整数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (uint32_t)getUnsignedInt;

/**
 * 64ビット整数値を取得します。
 * @return  現在位置から8バイトを64ビット整数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (int64_t)getInt64;

/**
 * 64ビット符号無し整数値を取得します。
 * @return  現在位置から8バイトを64ビット整数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (int64_t)getUnsignedInt64;

/**
 * 16ビット整数値を取得します。
 * @return  現在位置から2バイトを16ビット整数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (int16_t)getShort;

/**
 * 16ビット符号無し整数値を取得します。
 * @return  現在位置から2バイトを16ビット符号無し整数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (uint16_t)getUnsignedShort;

/**
 * 文字列を取得します。
 * @return  現在位置からNULL文字までをUTF-8文字列として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSString*)getString;

/**
 * 文字列を取得します。
 * @param   [in]    encoding    文字エンコーディング
 * @return  現在位置からNULL文字までを指定された文字エンコーディングでエンコードした文字列として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSString*)getStringUsingEncoding:(NSStringEncoding)encoding;

/**
 * 文字列を取得します。
 *
 * Windows版のCStreamXと同じ方式で文字列を取得します。
 * @return  現在位置からNULL文字までをUTF-8文字列として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSString*)getStringUsingStreamX;

/**
 * 文字列を取得します。
 *
 * Windows版のCStreamXと同じ方式で文字列を取得します。
 * @param   [in]    encoding    文字エンコーディング
 * @return  現在位置からNULL文字までを指定された文字エンコーディングでエンコードした文字列として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (NSString*)getStringUsingStreamXUsingEncoding:(NSStringEncoding)encoding;

/**
 * 倍精度浮動小数値を取得します。
 * @return  現在位置から8バイトを倍精度浮動小数値として返します。
 * @exception   NSException(IndexOutOfBoundsException)  読み取るバイト数が不足している場合
 */
- (double_t)getDouble;

/**
 * 現在位置を先頭に戻します。
 */
- (void)rewind;

/**
 * 現在位置を指定されたバイト数だけ進めます。
 * @param   [in]    bytes   進めるバイト数
 */
- (void)skip:(uint32_t)bytes;

/**
 * 処理可能なバイトが残っているかどうかテストします。
 * @retval  YES 残っている
 * @retval  NO  残っていない
 */
- (Boolean)hasRemain;

// 残りのデータを取得する
- (NSData*)remainData;

/**
 * 現在のバイト数を取得します。
 */
- (NSUInteger)length;

@end

/**
 * 様々なデータからバイトデータを構築するための機能を提供します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/08/12
 */
@interface MutableByteBuffer : NSObject {
    NSMutableData*  _source;    /**< 編集中のバイトデータ */
}

/**
 * 初期化します。
 * @return  自身を返します。
 */
- (id)init;

/**
 * Boolean値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putBoolean:(Boolean)value;

/**
 * Byte値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putByte:(uint8_t)value;

/**
 * バイトデータを追加します。
 * @param   [in]    value   追加する値
 */
- (void)putData:(NSData*)value;

/**
 * 1970/01/01 00:00:00からの経過時間をミリ秒単位で追加します。
 * @param   [in]    value   追加する値
 */
- (void)putDateSince1970WithMillis:(NSDate*)value;

/**
 * 1970/01/01 00:00:00からの経過時間を秒単位で追加します。
 * @param   [in]    value   追加する値
 */
- (void)putDateSince1970WithSeconds:(NSDate*)value;

/**
 * 32ビット整数値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putInt:(int32_t)value;

/**
 * Base128方式で32ビット整数値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putIntUsingBase128:(int32_t)value;

/**
 * 32ビット符号なし整数値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putUnsignedInt:(uint32_t)value;

/**
 * 64ビット整数値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putInt64:(int64_t)value;

/**
 * 64ビット符号無し整数値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putUnsignedInt64:(uint64_t)value;

/**
 * Boolean値を32ビット整数値として追加します。
 * @param   [in]    value   追加する値
 */
- (void)putIntWithBoolean:(Boolean)value;

/**
 * 16ビット整数値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putShort:(int16_t)value;

/**
 * 16ビット符号なし整数値を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putUnsignedShort:(uint16_t)value;

/**
 * 文字列を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putString:(NSString*)value;

/**
 * 文字列を追加します。
 * @param   [in]    value       追加する値
 * @param   [in]    encoding    文字エンコーディング
 */
- (void)putString:(NSString*)value encoding:(NSStringEncoding)encoding;

/**
 * 文字列を追加します。
 *
 * Windows版のCStreamXと同じ方式で文字列を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putStringUsingStreamX:(NSString*)value;

/**
 * 文字列を追加します。
 *
 * Windows版のCStreamXと同じ方式で文字列を追加します。
 * @param   [in]    value       追加する値
 * @param   [in]    encoding    文字エンコーディング
 */
- (void)putStringUsingStreamX:(NSString*)value encoding:(NSStringEncoding)encoding;

/**
 * 倍精度浮動小数を追加します。
 * @param   [in]    value   追加する値
 */
- (void)putDouble:(double_t)value;

/**
 * 編集中のデータを取得します。
 * @return  編集中のデータを返します。
 */
- (NSData*)data;

/*
 * 編集中のデータのコピーを返す
 */
- (NSMutableData*)mutableData;

/**
 * 現在のバイト数を取得します。
 */
- (NSUInteger)length;

@end

/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */
