/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */

#import <Foundation/Foundation.h>

/**
 * データを蓄えておくキューを操作するためのインターフェースを定義します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/10/04
 */
@protocol WaitableQueueProtocol <NSObject>

/**
 * 蓄えられている全てのデータを取得し、内部バッファを空にします。
 * プロキシ用
 * @return  保持していた全てのデータを返します。
 */
- (void)readQueueForProxy:(NSArray**)messages;

/**
 * 蓄えられている全てのデータを先頭から取得し、内部バッファを空にします。
 * @param   [in]    timeout データ取得のタイムアウト値(秒)
 * @param   [out]   buffer  保持している全てのデータを取得するためのバッファ
 * @retval  YES データが取得できた場合
 * @retval  NO  データが取得できなかった or タイムアウトした場合
 */
- (Boolean)readQueueWithTimeout:(NSTimeInterval)timeout intoBuffer:(NSArray**)buffer;

/**
 * データを追加します。
 * @param   [in]    data    追加したいデータ
 */
- (void)writeQueue:(id)data;

@end

/**
 * データを蓄えておくキューを操作するための機能を提供します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/10/04
 */
@interface WaitableQueue : NSObject<WaitableQueueProtocol> {
    NSMutableArray* _queue;     /**< 内部バッファ */
    NSCondition*    _queueLock; /**< 内部バッファの要素数に変更があったことを検知するためのイベント */
    
    NSArray* _readBufferCache;  /**< プロキシに渡すデータがガベージコレクトされないようにする為のキャッシュ */
}

/**
 * 初期化します。
 */
- (id)init;

/**
 * 蓄えられている全てのデータを取得し、内部バッファを空にします。
 * プロキシに渡すデータをキャッシュする
 * @return  保持していた全てのデータを返します。
 */
- (void)readQueueForProxy:(NSArray**)messages;

/**
 * 蓄えられている全てのデータを取得し、内部バッファを空にします。
 * @return  保持していた全てのデータを返します。
 */
- (void)readQueue:(NSArray**)messages;

/**
 * 蓄えられてるデータを指定数取得し、内部バッファを空にします。
 * @return  保持していた全てのデータを返します。
 */
- (NSArray*)readQueueWithCount:(NSUInteger)count;

/**
 * 蓄えられている全てのデータを取得し、内部バッファを空にします。
 * @param   [in]    timeout データ取得のタイムアウト値(秒)
 * @param   [out]   buffer  保持している全てのデータを取得するためのバッファ
 * @retval  YES データが取得できた場合
 * @retval  NO  データが取得できなかった or タイムアウトした場合
 */
- (Boolean)readQueueWithTimeout:(NSTimeInterval)timeout intoBuffer:(NSArray**)buffer;

/**
 * 指定した最大数のデータを先頭から取得し、内部バッファから削除します。
 * @param   [in]    timeout  データ取得のタイムアウト値(秒)
 * @param   [in]    maxCount 取得するデータの最大数
 * @param   [out]   buffer   保持しているデータを取得するためのバッファ
 * @retval  YES データが取得できた場合
 * @retval  NO  データが取得できなかった or タイムアウトした場合
 */
- (Boolean)readQueueWithTimeout:(NSTimeInterval)timeout maxCount:(NSUInteger)count intoBuffer:(NSArray**)buffer;

/**
 * 蓄えられている全てのデータを取得しします。。内部バッファはそのまま残ります。
 * @return  保持していた全てのデータを返します。
 */
- (NSArray*)queryQueue;

/**
 * 蓄えられてるデータを指定数取得します。。内部バッファはそのまま残ります。
 * @return  保持していた全てのデータを返します。
 */
- (NSArray*)queryQueueWithCount:(NSUInteger)count;

/**
 * 蓄えられている全てのデータを取得します。内部バッファはそのまま残ります。
 * @param   [in]    timeout データ取得のタイムアウト値(秒)
 * @param   [out]   buffer  保持している全てのデータを取得するためのバッファ
 * @retval  YES データが取得できた場合
 * @retval  NO  データが取得できなかった or タイムアウトした場合
 */
- (Boolean)queryQueueWithTimeout:(NSTimeInterval)timeout intoBuffer:(NSArray**)buffer;

/**
 * 指定した最大数のデータを取得します。内部バッファはそのまま残ります。
 * @param   [in]    timeout  データ取得のタイムアウト値(秒)
 * @param   [in]    maxCount 取得するデータの最大数
 * @param   [out]   buffer   保持しているデータを取得するためのバッファ
 * @retval  YES データが取得できた場合
 * @retval  NO  データが取得できなかった or タイムアウトした場合
 */
- (Boolean)queryQueueWithTimeout:(NSTimeInterval)timeout maxCount:(NSUInteger)count intoBuffer:(NSArray**)buffer;

/**
 * データを追加します。
 * @param   [in]    data    追加したいデータ
 */
- (void)writeQueue:(id)data;


/**
 * データをすべて削除します。
 */
- (void)removeAllQueue;

/**
 * データの数を取得します。
 */
- (NSUInteger)queueCount;

@end

/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */
